@@ -2,28 +2,68 @@
 # PYTHON_ARGCOMPLETE_OK
 
 import argparse
-import math
+import json
 import re
 import sys
+import time
 import argcomplete
+
 from simplyblock_core import cluster_ops, utils, db_controller
 from simplyblock_core import storage_node_ops as storage_ops
 from simplyblock_core import mgmt_node_ops as mgmt_ops
-from simplyblock_core import constants
 from simplyblock_core.controllers import pool_controller, lvol_controller, snapshot_controller, device_controller, \
     tasks_controller
 from simplyblock_core.controllers import caching_node_controller, health_controller
 from simplyblock_core.models.pool import Pool
+from simplyblock_core.models.cluster import Cluster
+
+
+def range_type(min, max):
+    def f(arg):
+        arg = int(arg)
+
+        if not (min <= arg < max):
+            raise argparse.ArgumentTypeError(f"Value '{arg}' must be in the interval [{min} {max})")
+
+        return arg
+
+    return f
+
+
+def size_type(min=None, max=None):
+    def f(arg):
+        size = utils.parse_size(arg)
+
+        if size == -1:
+            raise argparse.ArgumentTypeError(f"Invalid size '{arg}' passed")
+        elif min is not None and size < min:
+            raise argparse.ArgumentTypeError(f"Size must be larger than {utils.humanbytes(min)}")
+        elif max is not None and size > max:
+            raise argparse.ArgumentTypeError(f"Size must be smaller than {utils.humanbytes(max)}")
+
+        return size
+
+    return f
+
+
+def regex_type(regex):
+    def f(arg):
+        if (match := re.match(regex, arg)) is not None:
+            return match
+        else:
+            raise argparse.ArgumentTypeError(f"Argument '{arg}' invalid: does not match regex ({regex})")
+
+    return f
 
 
 class CLIWrapperBase:
 
     def __init__(self):
-        self.parser.add_argument("--cmd", help='cmd', nargs = '+')
+        self.parser.add_argument("--cmd", help='cmd', nargs='+')
         argcomplete.autocomplete(self.parser)
 
     def init_parser(self):
-        self.parser = argparse.ArgumentParser(prog=constants.SIMPLY_BLOCK_CLI_NAME, description='SimplyBlock management CLI')
+        self.parser = argparse.ArgumentParser(description='Simplyblock management CLI')
         self.parser.add_argument("-d", '--debug', help='Print debug messages', required=False, action='store_true')
         self.parser.add_argument('--dev', help='Enable developer options', required=False, action='store_true')
         self.subparser = self.parser.add_subparsers(dest='command')
@@ -38,27 +78,47 @@ class CLIWrapperBase:
         return parent_parser.add_parser(command, description=help, help=help, usage=usage)
 
     def storage_node__deploy(self, sub_command, args):
-        spdk_cpu_mask = None
-        if args.spdk_cpu_mask:
-            if self.validate_cpu_mask(args.spdk_cpu_mask):
-                spdk_cpu_mask = args.spdk_cpu_mask
-            else:
-                return f"Invalid cpu mask value: {args.spdk_cpu_mask}"
         isolate_cores = args.isolate_cores
-        return storage_ops.deploy(args.ifname, spdk_cpu_mask, isolate_cores)
+        return storage_ops.deploy(args.ifname, isolate_cores)
 
-    def storage_node__deploy_cleaner(self, sub_command, args):
-        return storage_ops.deploy_cleaner()
+    def storage_node__configure_upgrade(self, sub_command, args):
+        storage_ops.upgrade_automated_deployment_config()
 
-    def storage_node__add_node(self, sub_command, args):
+    def storage_node__configure(self, sub_command, args):
         if not args.max_lvol:
             self.parser.error(f"Mandatory argument '--max-lvol' not provided for {sub_command}")
         if not args.max_prov:
-            self.parser.error(f"Mandatory argument '--max-prov' not provided for {sub_command}")
-        # if not args.spdk_cpu_mask:
-        #     self.parser.error(f"Mandatory argument '--cpu-mask' not provided for {sub_command}")
+            self.parser.error(f"Mandatory argument '--max-size' not provided for {sub_command}")
+        sockets_to_use = [0]
+        if args.sockets_to_use:
+            try:
+                sockets_to_use = [int(x) for x in args.sockets_to_use.split(',')]
+            except ValueError:
+                self.parser.error(
+                        f"Invalid value for sockets_to_use {args.sockets_to_use}. It must be a comma-separated list of integers.")
+
+        if args.nodes_per_socket not in [1, 2]:
+            self.parser.error(f"nodes_per_socket {args.nodes_per_socket}must be either 1 or 2")
+        if args.pci_allowed and args.pci_blocked:
+            self.parser.error("pci-allowed and pci-blocked cannot be both specified")
+        max_prov = utils.parse_size(args.max_prov, assume_unit='G')
+        pci_allowed = []
+        pci_blocked = []
+        if args.pci_allowed:
+            pci_allowed = [str(x) for x in args.pci_allowed.split(',')]
+        if args.pci_blocked:
+            pci_blocked = [str(x) for x in args.pci_blocked.split(',')]
+
+        return storage_ops.generate_automated_deployment_config(args.max_lvol, max_prov, sockets_to_use,
+                                                                args.nodes_per_socket, pci_allowed, pci_blocked)
+
+    def storage_node__deploy_cleaner(self, sub_command, args):
+        storage_ops.deploy_cleaner()
+        return True  # remove once CLI changed to exceptions
+
+    def storage_node__add_node(self, sub_command, args):
         cluster_id = args.cluster_id
-        node_ip = args.node_ip
+        node_addr = args.node_addr
         ifname = args.ifname
         data_nics = args.data_nics
         spdk_image = args.spdk_image
@@ -68,53 +128,31 @@ class CLIWrapperBase:
         large_bufsize = args.large_bufsize
         num_partitions_per_dev = args.partitions
         jm_percent = args.jm_percent
-        spdk_cpu_mask = None
-        if args.spdk_cpu_mask:
-            if self.validate_cpu_mask(args.spdk_cpu_mask):
-                spdk_cpu_mask = args.spdk_cpu_mask
-            else:
-                return f"Invalid cpu mask value: {args.spdk_cpu_mask}"
 
-        max_lvol = args.max_lvol
         max_snap = args.max_snap
-        max_prov = utils.parse_size(args.max_prov, unit='G')
-        number_of_devices = args.number_of_devices
         enable_test_device = args.enable_test_device
         enable_ha_jm = args.enable_ha_jm
-        number_of_distribs = args.number_of_distribs
         namespace = args.namespace
         ha_jm_count = args.ha_jm_count
-        spdk_mem = args.spdk_mem
-        ssd_pcie = args.ssd_pcie
-        spdk_cpu_count = args.vcpu_count
 
         out = storage_ops.add_node(
             cluster_id=cluster_id,
-            node_ip=node_ip,
+            node_addr=node_addr,
             iface_name=ifname,
             data_nics_list=data_nics,
-            max_lvol=max_lvol,
             max_snap=max_snap,
-            max_prov=max_prov,
             spdk_image=spdk_image,
             spdk_debug=spdk_debug,
             small_bufsize=small_bufsize,
             large_bufsize=large_bufsize,
-            spdk_cpu_mask=spdk_cpu_mask,
             num_partitions_per_dev=num_partitions_per_dev,
             jm_percent=jm_percent,
-            number_of_devices=number_of_devices,
             enable_test_device=enable_test_device,
             namespace=namespace,
-            number_of_distribs=number_of_distribs,
             enable_ha_jm=enable_ha_jm,
-            is_secondary_node=args.is_secondary_node,   # pass
             id_device_by_nqn=args.id_device_by_nqn,
             partition_size=args.partition_size,
             ha_jm_count=ha_jm_count,
-            spdk_hp_mem=spdk_mem,
-            ssd_pcie=ssd_pcie,
-            spdk_cpu_count=spdk_cpu_count,
         )
 
         return out
@@ -136,19 +174,21 @@ class CLIWrapperBase:
 
         spdk_image = args.spdk_image
         spdk_debug = args.spdk_debug
+        reattach_volume = args.reattach_volume
 
         max_lvol = args.max_lvol
         max_snap = args.max_snap
-        max_prov = utils.parse_size(args.max_prov, unit='G')
-        number_of_devices = args.number_of_devices
+        max_prov = utils.parse_size(args.max_prov)
 
         small_bufsize = args.small_bufsize
         large_bufsize = args.large_bufsize
+        ssd_pcie = args.ssd_pcie
 
         return storage_ops.restart_storage_node(
             node_id, max_lvol, max_snap, max_prov,
             spdk_image, spdk_debug,
-            small_bufsize, large_bufsize, number_of_devices, node_ip=args.node_ip, force=args.force)
+            small_bufsize, large_bufsize, node_ip=args.node_ip, reattach_volume=reattach_volume, force=args.force,
+            new_ssd_pcie=ssd_pcie)
 
     def storage_node__shutdown(self, sub_command, args):
         return storage_ops.shutdown_storage_node(args.node_id, args.force)
@@ -171,13 +211,13 @@ class CLIWrapperBase:
             return False
 
     def storage_node__get_capacity(self, sub_command, args):
-                node_id = args.node_id
-                history = args.history
-                data = storage_ops.get_node_capacity(node_id, history)
-                if data:
-                    return utils.print_table(data)
-                else:
-                    return False
+        node_id = args.node_id
+        history = args.history
+        data = storage_ops.get_node_capacity(node_id, history)
+        if data:
+            return utils.print_table(data)
+        else:
+            return False
 
     def storage_node__list_devices(self, sub_command, args):
         return self.storage_node_list_devices(args)
@@ -271,10 +311,7 @@ class CLIWrapperBase:
         return storage_ops.dump_lvstore(node_id)
 
     def storage_node__set(self, sub_command, args):
-        return storage_ops.set(args.node_id, args.attr_name, args.attr_value)
-
-    def cluster__deploy(self, sub_command, args):
-        return self.cluster_deploy(args)
+        return storage_ops.set_value(args.node_id, args.attr_name, args.attr_value)
 
     def cluster__create(self, sub_command, args):
         return self.cluster_create(args)
@@ -283,95 +320,124 @@ class CLIWrapperBase:
         return self.cluster_add(args)
 
     def cluster__activate(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.cluster_activate(cluster_id, args.force, args.force_lvstore_create)
+        cluster_ops.cluster_activate(args.cluster_id, args.force, args.force_lvstore_create)
+        return True
 
     def cluster__list(self, sub_command, args):
-        return cluster_ops.list()
+        data = cluster_ops.list()
+
+        if args.json:
+            return json.dumps(data, indent=2)
+        else:
+            return utils.print_table(data)
 
     def cluster__status(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.get_cluster_status(cluster_id)
+        return utils.print_table(cluster_ops.get_cluster_status(args.cluster_id))
 
     def cluster__show(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.list_all_info(cluster_id)
+        return cluster_ops.list_all_info(args.cluster_id)
 
     def cluster__get(self, sub_command, args):
-        return cluster_ops.get_cluster(args.cluster_id)
+        return json.dumps(cluster_ops.get_cluster(args.cluster_id), indent=2, sort_keys=True)
 
     def cluster__get_capacity(self, sub_command, args):
-        cluster_id = args.cluster_id
-        history = args.history
         is_json = args.json
-        data = cluster_ops.get_capacity(cluster_id, history, is_json=is_json)
+        data = cluster_ops.get_capacity(args.cluster_id, args.history)
+
         if is_json:
-            return data
+            return json.dumps(data, indent=2)
         else:
-            return utils.print_table(data)
+            return utils.print_table([
+                {
+                    "Date": time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(record['date'])),
+                    "Absolut": utils.humanbytes(record['size_total']),
+                    "Provisioned": utils.humanbytes(record['size_prov']),
+                    "Used": utils.humanbytes(record['size_used']),
+                    "Free": utils.humanbytes(record['size_free']),
+                    "Util %": f"{record['size_util']}%",
+                    "Prov Util %": f"{record['size_prov_util']}%",
+                }
+                for record in data
+            ])
 
     def cluster__get_io_stats(self, sub_command, args):
-        data = cluster_ops.get_iostats_history(args.cluster_id, args.history, args.records)
-        if data:
-            return utils.print_table(data)
-        else:
-            return False
+        return utils.print_table([
+            {
+                "Date": time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(record['date'])),
+                "Read speed": utils.humanbytes(record['read_bytes_ps']),
+                "Read IOPS": record["read_io_ps"],
+                "Read lat": record["read_latency_ps"],
+                "Write speed": utils.humanbytes(record["write_bytes_ps"]),
+                "Write IOPS": record["write_io_ps"],
+                "Write lat": record["write_latency_ps"],
+            }
+            for record in cluster_ops.get_iostats_history(args.cluster_id, args.history, args.records)
+        ])
 
     def cluster__get_logs(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.get_logs(cluster_id)
+        cluster_logs = cluster_ops.get_logs(**args.__dict__)
+
+        if args.json:
+            return json.dumps(cluster_logs, indent=2)
+        else:
+            return utils.print_table(cluster_logs)
 
     def cluster__get_secret(self, sub_command, args):
         cluster_id = args.cluster_id
         return cluster_ops.get_secret(cluster_id)
 
     def cluster__update_secret(self, sub_command, args):
-        cluster_id = args.cluster_id
-        secret = args.secret
-        return cluster_ops.set_secret(cluster_id, secret)
+        cluster_ops.set_secret(args.cluster_id, args.secret)
+        return True
 
     def cluster__check(self, sub_command, args):
         cluster_id = args.cluster_id
         return health_controller.check_cluster(cluster_id)
 
     def cluster__update(self, sub_command, args):
-        return cluster_ops.update_cluster(args.cluster_id, mgmt_only=args.mgmt_only, restart_cluster=args.restart)
-    
+        cluster_ops.update_cluster(**args.__dict__)
+        return True
+
     def cluster__graceful_shutdown(self, sub_command, args):
-        return cluster_ops.cluster_grace_shutdown(args.cluster_id)
+        cluster_ops.cluster_grace_shutdown(args.cluster_id)
+        return True
 
     def cluster__graceful_startup(self, sub_command, args):
-        return cluster_ops.cluster_grace_startup(args.cluster_id, args.clear_data, args.spdk_image)
+        cluster_ops.cluster_grace_startup(args.cluster_id, args.clear_data, args.spdk_image)
+        return True
 
     def cluster__list_tasks(self, sub_command, args):
-        return tasks_controller.list_tasks(args.cluster_id)
+        return tasks_controller.list_tasks(**args.__dict__)
 
     def cluster__cancel_task(self, sub_command, args):
         return tasks_controller.cancel_task(args.task_id)
 
     def cluster__delete(self, sub_command, args):
-        return cluster_ops.delete_cluster(args.cluster_id)
-    
-    def cluster_suspend(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.suspend_cluster(cluster_id) 
-    
+        cluster_ops.delete_cluster(args.cluster_id)
+        return True
+
+    def cluster__suspend(self, sub_command, args):
+        return cluster_ops.set_cluster_status(args.cluster_id, Cluster.STATUS_SUSPENDED)
+
     def cluster_unsuspend(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.unsuspend_cluster(cluster_id) 
-    
+        return cluster_ops.set_cluster_status(args.cluster_id, Cluster.STATUS_ACTIVE)
+
     def cluster_get_cli_ssh_pass(self, sub_command, args):
         cluster_id = args.cluster_id
         return cluster_ops.get_ssh_pass(cluster_id)
 
     def cluster__set(self, sub_command, args):
-        cluster_id = args.cluster_id
-        return cluster_ops.set(cluster_id,  args.attr_name, args.attr_value)
+        cluster_ops.set(args.cluster_id, args.attr_name, args.attr_value)
+        return True
+
+    def cluster__complete_expand(self, sub_command, args):
+        cluster_ops.cluster_expand(args.cluster_id)
+        return True
 
     def volume__add(self, sub_command, args):
         name = args.name
-        size = utils.parse_size(args.size)
-        max_size = utils.parse_size(args.max_size)
+        size = args.size
+        max_size = args.max_size
         host_id = args.host_id
         ha_type = args.ha_type
         pool = args.pool
@@ -418,15 +484,18 @@ class CLIWrapperBase:
             return lvol_controller.delete_lvol(id, force)
 
     def volume__connect(self, sub_command, args):
-        volume_id = args.volume_id
-        data = lvol_controller.connect_lvol(volume_id)
+        kwargs = {}
+        if (ctrl_loss_tmo := args.ctrl_loss_tmo) is not None:
+            kwargs['ctrl_loss_tmo'] = ctrl_loss_tmo
+
+        data = lvol_controller.connect_lvol(args.volume_id, **kwargs)
         if data:
             return "\n".join(con['connect'] for con in data)
 
     def volume__resize(self, sub_command, args):
         volume_id = args.volume_id
-        size = utils.parse_size(args.size)
-        ret, err =  lvol_controller.resize_lvol(volume_id, size)
+        size = args.size
+        ret, err = lvol_controller.resize_lvol(volume_id, size)
         return ret
 
     def volume__create_snapshot(self, sub_command, args):
@@ -436,9 +505,7 @@ class CLIWrapperBase:
         return snapshot_id if not error else error
 
     def volume__clone(self, sub_command, args):
-        new_size = 0
-        if args.resize:
-            new_size = utils.parse_size(args.resize)
+        new_size = args.resize
 
         clone_id, error = snapshot_controller.clone(args.snapshot_id, args.clone_name, new_size)
         return clone_id if not error else error
@@ -491,8 +558,8 @@ class CLIWrapperBase:
             has_secret = False
         return pool_controller.add_pool(
             args.name,
-            utils.parse_size(args.pool_max),
-            utils.parse_size(args.lvol_max),
+            args.pool_max,
+            args.lvol_max,
             args.max_rw_iops,
             args.max_rw_mbytes,
             args.max_r_mbytes,
@@ -502,13 +569,10 @@ class CLIWrapperBase:
         )
 
     def storage_pool__set(self, sub_command, args):
-        pool_max = None
-        lvol_max = None
-        if args.pool_max:
-            pool_max = utils.parse_size(args.pool_max)
-        if args.lvol_max:
-            lvol_max = utils.parse_size(args.lvol_max)
-        return pool_controller.set_pool(
+        pool_max = args.pool_max
+        lvol_max = args.lvol_max
+
+        ret, err = pool_controller.set_pool(
             args.pool_id,
             pool_max,
             lvol_max,
@@ -516,6 +580,7 @@ class CLIWrapperBase:
             args.max_rw_mbytes,
             args.max_r_mbytes,
             args.max_w_mbytes)
+        return ret
 
     def storage_pool__list(self, sub_command, args):
         return pool_controller.list_pools(args.json, args.cluster_id)
@@ -555,9 +620,7 @@ class CLIWrapperBase:
         return snapshot_controller.delete(args.snapshot_id, args.force)
 
     def snapshot__clone(self, sub_command, args):
-        new_size = 0
-        if args.resize:
-            new_size = utils.parse_size(args.resize)
+        new_size = args.resize
 
         success, details = snapshot_controller.clone(args.snapshot_id, args.lvol_name, new_size)
         return details
@@ -573,19 +636,8 @@ class CLIWrapperBase:
         spdk_image = args.spdk_image
         namespace = args.namespace
         multipathing = args.multipathing == "on"
-
-        spdk_cpu_mask = None
-        if args.spdk_cpu_mask:
-            if self.validate_cpu_mask(args.spdk_cpu_mask):
-                spdk_cpu_mask = args.spdk_cpu_mask
-            else:
-                return f"Invalid cpu mask value: {args.spdk_cpu_mask}"
-
-        spdk_mem = None
-        if args.spdk_mem:
-            spdk_mem = utils.parse_size(args.spdk_mem)
-            if spdk_mem < utils.parse_size('1GiB'):
-                return f"SPDK memory:{args.spdk_mem} must be larger than 1GiB"
+        spdk_cpu_mask = args.spdk_cpu_mask
+        spdk_mem = args.spdk_mem
 
         return caching_node_controller.add_node(
             cluster_id, node_ip, ifname, data_nics, spdk_cpu_mask, spdk_mem, spdk_image, namespace, multipathing)
@@ -641,110 +693,11 @@ class CLIWrapperBase:
         enable_qos = args.enable_qos
         strict_node_anti_affinity = args.strict_node_anti_affinity
 
-
         return cluster_ops.add_cluster(
             blk_size, page_size_in_blocks, cap_warn, cap_crit, prov_cap_warn, prov_cap_crit,
             distr_ndcs, distr_npcs, distr_bs, distr_chunk_bs, ha_type, enable_node_affinity,
             qpair_count, max_queue_size, inflight_io_threshold, enable_qos, strict_node_anti_affinity)
 
-
-    def cluster_deploy(self,args):
-        grafana_endpoint = ""
-        secondary_nodes = False
-        namespace = None
-        lvol_name = "lvol01"
-        lvol_size = self.parse_size("10G")
-        pool_max = self.parse_size("25G")
-        max_size = self.parse_size("1000G")
-        pool_name = "pool01"
-        with_snapshot = False
-        host_id = None
-        crypto = False
-        crypto_key1 = None
-        crypto_key2 = None
-        max_rw_iops = None
-        max_rw_mbytes = None
-        max_r_mbytes = None
-        max_w_mbytes = None
-        lvol_priority_class = 0
-        id_device_by_nqn = False
-        fstype = "xfs"
-
-        storage_nodes = args.storage_nodes
-        test = args.test
-        ha_type = args.ha_type
-        ha_jm_count = args.ha_jm_count
-        distr_ndcs = args.distr_ndcs
-        distr_npcs = args.distr_npcs
-        enable_qos = args.enable_qos
-        ifname = args.ifname
-        page_size_in_blocks = args.page_size
-        blk_size = args.blk_size
-        CLI_PASS = args.CLI_PASS
-        cap_warn = args.cap_warn
-        cap_crit = args.cap_crit
-        prov_cap_warn = args.prov_cap_warn
-        prov_cap_crit = args.prov_cap_crit
-        distr_bs = args.distr_bs
-        distr_chunk_bs = args.distr_chunk_bs
-        log_del_interval = args.log_del_interval
-        metrics_retention_period = args.metrics_retention_period
-        contact_point = args.contact_point
-        enable_node_affinity = args.enable_node_affinity
-        qpair_count = args.qpair_count
-        max_queue_size = args.max_queue_size
-        inflight_io_threshold = args.inflight_io_threshold
-        strict_node_anti_affinity = args.strict_node_anti_affinity
-        
-        data_nics = args.data_nics
-        spdk_image = args.spdk_image
-        spdk_debug = args.spdk_debug
-
-        small_bufsize = args.small_bufsize
-        large_bufsize = args.large_bufsize
-        num_partitions_per_dev = args.partitions
-        partition_size = args.partition_size
-        jm_percent = args.jm_percent
-        spdk_cpu_mask = None
-        if args.spdk_cpu_mask:
-            if self.validate_cpu_mask(args.spdk_cpu_mask):
-                spdk_cpu_mask = args.spdk_cpu_mask
-            else:
-                return f"Invalid cpu mask value: {args.spdk_cpu_mask}"
-
-        
-
-        max_lvol = args.max_lvol
-        max_snap = args.max_snap
-        max_prov = utils.parse_size(args.max_prov, unit='G')
-        number_of_devices = args.number_of_devices
-        enable_test_device = args.enable_test_device
-        enable_ha_jm = args.enable_ha_jm
-        number_of_distribs = args.number_of_distribs
-        namespace = args.namespace
-        secondary_nodes = args.secondary_nodes
-        
-        lvol_name = args.lvol_name
-        lvol_size = utils.parse_size(args.lvol_size)
-        max_size = utils.parse_size(args.max_size)
-        lvol_ha_type = args.lvol_ha_type
-        pool_name = args.pool_name
-        pool_max = utils.parse_size(args.pool_max)
-        host_id = args.host_id
-        comp = None
-        distr_vuid = args.distr_vuid
-
-        return cluster_ops.deploy_cluster(
-            storage_nodes,test,ha_type,distr_ndcs,distr_npcs,enable_qos,ifname,
-            blk_size, page_size_in_blocks,CLI_PASS, cap_warn, cap_crit, prov_cap_warn, 
-            prov_cap_crit,log_del_interval, metrics_retention_period, contact_point, grafana_endpoint,
-            distr_bs, distr_chunk_bs, enable_node_affinity,
-            qpair_count, max_queue_size, inflight_io_threshold, strict_node_anti_affinity,data_nics,
-            spdk_image,spdk_debug,small_bufsize,large_bufsize,num_partitions_per_dev,jm_percent,spdk_cpu_mask,max_lvol,
-            max_snap,max_prov,number_of_devices,enable_test_device,enable_ha_jm,ha_jm_count,number_of_distribs,namespace,secondary_nodes,partition_size,
-            lvol_name, lvol_size, lvol_ha_type, pool_name, pool_max, host_id, comp, crypto, distr_vuid, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes,
-            with_snapshot, max_size, crypto_key1, crypto_key2, lvol_priority_class, id_device_by_nqn, fstype)
-
     def cluster_create(self, args):
         page_size_in_blocks = args.page_size
         blk_size = 4096
@@ -768,16 +721,15 @@ class CLIWrapperBase:
         max_queue_size = args.max_queue_size
         inflight_io_threshold = args.inflight_io_threshold
         enable_qos = args.enable_qos
+        disable_monitoring = args.disable_monitoring
         strict_node_anti_affinity = args.strict_node_anti_affinity
 
-
         return cluster_ops.create_cluster(
             blk_size, page_size_in_blocks,
             CLI_PASS, cap_warn, cap_crit, prov_cap_warn, prov_cap_crit,
             ifname, log_del_interval, metrics_retention_period, contact_point, grafana_endpoint,
             distr_ndcs, distr_npcs, distr_bs, distr_chunk_bs, ha_type, enable_node_affinity,
-            qpair_count, max_queue_size, inflight_io_threshold, enable_qos, strict_node_anti_affinity)
-
+            qpair_count, max_queue_size, inflight_io_threshold, enable_qos, disable_monitoring, strict_node_anti_affinity)
 
     def query_yes_no(self, question, default="yes"):
         """Ask a yes/no question via raw_input() and return their answer.
@@ -809,14 +761,10 @@ class CLIWrapperBase:
             else:
                 sys.stdout.write("Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n")
 
-    def validate_cpu_mask(self, spdk_cpu_mask):
-        return re.match("^(0x|0X)?[a-fA-F0-9]+$", spdk_cpu_mask)
-
     def _completer_get_cluster_list(self, prefix, parsed_args, **kwargs):
         db = db_controller.DBController()
         return (cluster.get_id() for cluster in db.get_clusters() if cluster.get_id().startswith(prefix))
 
-
     def _completer_get_sn_list(self, prefix, parsed_args, **kwargs):
         db = db_controller.DBController()
         return (cluster.get_id() for cluster in db.get_storage_nodes() if cluster.get_id().startswith(prefix))
