@@ -1,8 +1,6 @@
 # coding=utf-8
-import datetime
 import json
 import logging as lg
-import math
 import time
 import uuid
 
@@ -14,70 +12,23 @@ from simplyblock_core.cnode_client import CNodeClient
 from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.caching_node import CachingNode, CachedLVol
 from simplyblock_core.models.iface import IFace
-from simplyblock_core.models.nvme_device import NVMeDevice
 from simplyblock_core.models.pool import Pool
+from simplyblock_core.utils import addNvmeDevices
 from simplyblock_core.rpc_client import RPCClient
+from simplyblock_core.utils import pull_docker_image_with_retry
 
 logger = lg.getLogger()
 
 db_controller = DBController()
 
 
-def addNvmeDevices(rpc_client, devs, snode):
-    devices = []
-    ret = rpc_client.bdev_nvme_controller_list()
-    ctr_map = {}
-    try:
-        if ret:
-            ctr_map = {i["ctrlrs"][0]['trid']['traddr']: i["name"] for i in ret}
-    except:
-        pass
-
-    for index, pcie in enumerate(devs):
-        if pcie in ctr_map:
-            nvme_controller = ctr_map[pcie]
-            nvme_bdevs = []
-            for bdev in rpc_client.get_bdevs():
-                if bdev['name'].startswith(nvme_controller):
-                    nvme_bdevs.append(bdev['name'])
-        else:
-            pci_st = str(pcie).replace("0", "").replace(":", "").replace(".", "")
-            nvme_controller = "nvme_%s" % pci_st
-            nvme_bdevs, err = rpc_client.bdev_nvme_controller_attach(nvme_controller, pcie)
-            time.sleep(2)
-
-        for nvme_bdev in nvme_bdevs:
-            rpc_client.bdev_examine(nvme_bdev)
-            time.sleep(3)
-            ret = rpc_client.get_bdevs(nvme_bdev)
-            nvme_dict = ret[0]
-            nvme_driver_data = nvme_dict['driver_specific']['nvme'][0]
-            model_number = nvme_driver_data['ctrlr_data']['model_number']
-            total_size = nvme_dict['block_size'] * nvme_dict['num_blocks']
-
-            devices.append(
-                NVMeDevice({
-                    'uuid': str(uuid.uuid4()),
-                    'device_name': nvme_dict['name'],
-                    'size': total_size,
-                    'pcie_address': nvme_driver_data['pci_address'],
-                    'model_id': model_number,
-                    'serial_number': nvme_driver_data['ctrlr_data']['serial_number'],
-                    'nvme_bdev': nvme_bdev,
-                    'nvme_controller': nvme_controller,
-                    'node_id': snode.get_id(),
-                    'cluster_id': snode.cluster_id,
-                    'status': NVMeDevice.STATUS_ONLINE
-                }))
-    return devices
-
-
 def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spdk_mem, spdk_image=None, namespace=None, multipathing=True):
     db_controller = DBController()
     kv_store = db_controller.kv_store
 
-    cluster = db_controller.get_cluster_by_id(cluster_id)
-    if not cluster:
+    try:
+        cluster = db_controller.get_cluster_by_id(cluster_id)
+    except KeyError:
         logger.error("Cluster not found: %s", cluster_id)
         return False
 
@@ -135,6 +86,7 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
     snode.multipathing = multipathing
 
     # check for memory
+    spdk_mem_min = utils.parse_size('1GiB')
     if "memory_details" in node_info and node_info['memory_details']:
         memory_details = node_info['memory_details']
         logger.info("Node Memory info")
@@ -144,13 +96,11 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
         logger.info(f"HP Total: {utils.humanbytes(huge_total)}")
         huge_free = memory_details['huge_free']
         logger.info(f"HP Free: {utils.humanbytes(huge_free)}")
-        # if huge_free < 1 * 1024 * 1024:
-        #     logger.warning(f"Free hugepages are less than 1G: {utils.humanbytes(huge_free)}")
         if not spdk_mem:
-            if huge_total > 1024 * 1024 * 1024:
+            if huge_total > spdk_mem_min:
                 spdk_mem = huge_total
             else:
-                logger.error(f"Free hugepages are less than 1G: {utils.humanbytes(huge_total)}")
+                logger.error(f"Free hugepages are less than {utils.humanbytes(spdk_mem_min)}: {utils.humanbytes(huge_total)}")
                 return False
 
     logger.info(f"Deploying SPDK with HP: {utils.humanbytes(spdk_mem)}")
@@ -165,7 +115,7 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
     while retries > 0:
         resp, _ = cnode_api.spdk_process_is_up()
         if resp:
-            logger.info(f"Pod is up")
+            logger.info("Pod is up")
             break
         else:
             logger.info("Pod is not running, waiting...")
@@ -176,7 +126,12 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
         logger.error("Pod is not running, exiting")
         return False
 
-    time.sleep(20)
+    time.sleep(3)
+
+    snode.namespace = namespace
+    snode.spdk_cpu_mask = spdk_cpu_mask
+    snode.spdk_mem = spdk_mem
+    snode.spdk_image = spdk_image
 
     # creating RPCClient instance
     rpc_client = RPCClient(
@@ -190,7 +145,7 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
     # logger.info(f"Free Hugepages detected: {utils.humanbytes(mem)}")
 
     # adding devices
-    nvme_devs = addNvmeDevices(rpc_client, node_info['spdk_pcie_list'], snode)
+    nvme_devs = addNvmeDevices(rpc_client, snode, node_info['spdk_pcie_list'])
     if not nvme_devs:
         logger.error("No NVMe devices was found!")
         return False
@@ -199,76 +154,75 @@ def add_node(cluster_id, node_ip, iface_name, data_nics_list, spdk_cpu_mask, spd
     snode.write_to_db(db_controller.kv_store)
     ssd_dev = nvme_devs[0]
 
-    if spdk_mem < 1024*1024:
-        logger.error("Hugepages must be larger than 1G")
+    if spdk_mem < spdk_mem_min:
+        logger.error(f"Hugepages must be larger than {utils.humanbytes(spdk_mem_min)}")
         return False
 
-    mem = spdk_mem - 1024*1024*1024
+    mem = spdk_mem - spdk_mem_min
     snode.hugepages = mem
     logger.info(f"Hugepages to be used: {utils.humanbytes(mem)}")
 
     ssd_size = ssd_dev.size
     supported_ssd_size = mem * 100 / 2.25
-    split_factor = math.ceil(ssd_size/supported_ssd_size)
 
     logger.info(f"Supported SSD size: {utils.humanbytes(supported_ssd_size)}")
     logger.info(f"SSD size: {utils.humanbytes(ssd_size)}")
 
     cache_size = 0
     cache_bdev = None
-    if supported_ssd_size < ssd_size:
-        logger.info(f"SSD size is bigger than the supported size, will use split bdev: {split_factor}")
-        ret = rpc_client.bdev_split(ssd_dev.nvme_bdev, split_factor)
-        cache_bdev = ret[0]
-        cache_size = int(ssd_dev.size/split_factor)
-        snode.cache_split_factor = split_factor
-    else:
-        snode.cache_split_factor = 0
-        cache_bdev = ssd_dev.nvme_bdev
-        cache_size = ssd_dev.size
-
     # if supported_ssd_size < ssd_size:
-    #     logger.info(f"SSD size is bigger than the supported size, creating partition")
-    #
-    #     nbd_device = rpc_client.nbd_start_disk(ssd_dev.nvme_bdev)
-    #     time.sleep(3)
-    #     if not nbd_device:
-    #         logger.error(f"Failed to start nbd dev")
-    #         return False
-    #
-    #     jm_percent = int((supported_ssd_size/ssd_size) * 100)
-    #     result, error = cnode_api.make_gpt_partitions(nbd_device, jm_percent)
-    #     if error:
-    #         logger.error(f"Failed to make partitions")
-    #         logger.error(error)
-    #         return False
-    #     time.sleep(3)
-    #     rpc_client.nbd_stop_disk(nbd_device)
-    #     time.sleep(1)
-    #     rpc_client.bdev_nvme_detach_controller(ssd_dev.nvme_controller)
-    #     time.sleep(1)
-    #     rpc_client.bdev_nvme_controller_attach(ssd_dev.nvme_controller, ssd_dev.pcie_address)
-    #     time.sleep(1)
-    #     rpc_client.bdev_examine(ssd_dev.nvme_bdev)
-    #     time.sleep(1)
-    #
-    #     cache_bdev = f"{ssd_dev.nvme_bdev}p1"
-    #     cache_size = int(supported_ssd_size)
-    #
+    #     logger.info(f"SSD size is bigger than the supported size, will use split bdev: {split_factor}")
+    #     ret = rpc_client.bdev_split(ssd_dev.nvme_bdev, split_factor)
+    #     cache_bdev = ret[0]
+    #     cache_size = int(ssd_dev.size/split_factor)
+    #     snode.cache_split_factor = split_factor
     # else:
-    #
+    #     snode.cache_split_factor = 0
     #     cache_bdev = ssd_dev.nvme_bdev
     #     cache_size = ssd_dev.size
 
+    if supported_ssd_size < ssd_size:
+        logger.info("SSD size is bigger than the supported size, creating partition")
+
+        nbd_device = rpc_client.nbd_start_disk(ssd_dev.nvme_bdev)
+        time.sleep(3)
+        if not nbd_device:
+            logger.error("Failed to start nbd dev")
+            return False
+
+        jm_percent = int((supported_ssd_size/ssd_size) * 100)
+        result, error = cnode_api.make_gpt_partitions(nbd_device, jm_percent)
+        if error:
+            logger.error("Failed to make partitions")
+            logger.error(error)
+            return False
+        time.sleep(3)
+        rpc_client.nbd_stop_disk(nbd_device)
+        time.sleep(1)
+        rpc_client.bdev_nvme_detach_controller(ssd_dev.nvme_controller)
+        time.sleep(1)
+        rpc_client.bdev_nvme_controller_attach(ssd_dev.nvme_controller, ssd_dev.pcie_address)
+        time.sleep(1)
+        rpc_client.bdev_examine(ssd_dev.nvme_bdev)
+        time.sleep(1)
+
+        cache_bdev = f"{ssd_dev.nvme_bdev}p1"
+        cache_size = int(supported_ssd_size)
+
+    else:
+
+        cache_bdev = ssd_dev.nvme_bdev
+        cache_size = ssd_dev.size
+
     logger.info(f"Cache size: {utils.humanbytes(cache_size)}")
 
     snode.cache_bdev = cache_bdev
     snode.cache_size = cache_size
 
     # create tmp ocf
-    logger.info(f"Creating first ocf bdev...")
+    logger.info("Creating first ocf bdev...")
 
-    ret = rpc_client.bdev_malloc_create("malloc_tmp", 512, int((100*1024*1024)/512))
+    ret = rpc_client.bdev_malloc_create("malloc_tmp", 512, int(utils.parse_size('100MiB')/512))
     if not ret:
         logger.error("Failed ot create tmp malloc")
         return False
@@ -291,13 +245,39 @@ def recreate(node_id):
         logger.error(f"Can not find caching node: {node_id}")
         return False
 
-    cluster = db_controller.get_cluster_by_id(snode.cluster_id)
-    if not cluster:
+    try:
+        db_controller.get_cluster_by_id(snode.cluster_id)
+    except KeyError:
         logger.error("Cluster not found: %s", snode.cluster_id)
         return False
 
     logger.info(f"Recreating caching node: {node_id}, status: {snode.status}")
-    snode_api = CNodeClient(f"{snode.mgmt_ip}:5000")
+    cnode_api = CNodeClient(f"{snode.mgmt_ip}:5000")
+
+    if not cnode_api.spdk_process_is_up():
+        results, err = cnode_api.spdk_process_start(
+            snode.spdk_cpu_mask, snode.spdk_mem, snode.spdk_image, snode.mgmt_ip,
+            snode.rpc_port, snode.rpc_username, snode.rpc_password, snode.namespace)
+        if not results:
+            logger.error(f"Failed to start spdk: {err}")
+            return False
+
+        retries = 20
+        while retries > 0:
+            resp, _ = cnode_api.spdk_process_is_up()
+            if resp:
+                logger.info("Pod is up")
+                break
+            else:
+                logger.info("Pod is not running, waiting...")
+                time.sleep(3)
+                retries -= 1
+
+        if retries == 0:
+            logger.error("Pod is not running, exiting")
+            return False
+
+        time.sleep(5)
 
     # creating RPCClient instance
     rpc_client = RPCClient(
@@ -306,7 +286,7 @@ def recreate(node_id):
         timeout=60*5, retry=5)
 
     # get new node info after starting spdk
-    node_info, _ = snode_api.info()
+    node_info, _ = cnode_api.info()
     # adding devices
     nvme_devs = addNvmeDevices(rpc_client, node_info['spdk_pcie_list'], snode)
     if not nvme_devs:
@@ -324,9 +304,9 @@ def recreate(node_id):
     logger.info(f"Cache size: {utils.humanbytes(snode.cache_size)}")
 
     # create tmp ocf
-    logger.info(f"Creating first ocf bdev...")
+    logger.info("Creating first ocf bdev...")
 
-    ret = rpc_client.bdev_malloc_create("malloc_tmp", 512, int((100 * 1024 * 1024) / 512))
+    ret = rpc_client.bdev_malloc_create("malloc_tmp", 512, int(utils.parse_size('100MiB') / 512))
     if not ret:
         logger.error("Failed ot create tmp malloc")
         return False
@@ -337,7 +317,7 @@ def recreate(node_id):
 
     if snode.lvols:
         for lvol in snode.lvols:
-            ret = connect(snode.get_id(), lvol.lvol_id)
+            ret = connect(snode.get_id(), lvol.lvol_id, force=True)
             if ret:
                 logger.info(f"connecting lvol {lvol.lvol_id} ... ok")
             else:
@@ -351,7 +331,7 @@ def recreate(node_id):
     return True
 
 
-def connect(caching_node_id, lvol_id):
+def connect(caching_node_id, lvol_id, force=False):
     lvol = db_controller.get_lvol_by_id(lvol_id)
     if not lvol:
         logger.error(f"LVol not found: {lvol_id}")
@@ -363,7 +343,7 @@ def connect(caching_node_id, lvol_id):
 
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
         return False
 
     cnode = None
@@ -383,7 +363,8 @@ def connect(caching_node_id, lvol_id):
     for clvol in cnode.lvols:
         if clvol.lvol_id == lvol_id:
             logger.info(f"Already connected, dev path: {clvol.device_path}")
-            return False
+            if not force:
+                return False
 
     if cnode.cluster_id != pool.cluster_id:
         logger.error("Caching node and LVol are in different clusters")
@@ -423,7 +404,8 @@ def connect(caching_node_id, lvol_id):
     logger.debug(ret)
     if not ret:
         logger.error("Failed to create OCF bdev")
-        return False
+        if not force:
+            return False
 
     # logger.info("Creating local subsystem")
     # create subsystem (local)
@@ -443,7 +425,8 @@ def connect(caching_node_id, lvol_id):
 
     if not ret:
         logger.error(f"Failed to add: {cach_bdev} to the subsystem: {subsystem_nqn}")
-        return False
+        if not force:
+            return False
 
     logger.info("Connecting to local subsystem")
     # make nvme connect to nqn
@@ -451,7 +434,8 @@ def connect(caching_node_id, lvol_id):
     ret, _ = cnode_client.connect_nvme('127.0.0.1', "4420", subsystem_nqn)
     if not ret:
         logger.error("Failed to connect to local subsystem")
-        return False
+        if not force:
+            return False
 
     if cnode.multipathing:
         snode = db_controller.get_storage_node_by_id(lvol.node_id)
@@ -460,7 +444,7 @@ def connect(caching_node_id, lvol_id):
             ret, _ = cnode_client.connect_nvme(ip, "4420", subsystem_nqn)
             break
 
-    time.sleep(5)
+    time.sleep(3)
     cnode_info, _ = cnode_client.info()
     nvme_devs = cnode_info['nvme_devices']
     dev_path = None
@@ -470,7 +454,7 @@ def connect(caching_node_id, lvol_id):
             break
 
     if not dev_path:
-        logger.error(f"Device path was not found")
+        logger.error("Device path was not found")
         return False
 
     logger.info(f"Device path: {dev_path}")
@@ -533,7 +517,7 @@ def disconnect(caching_node_id, lvol_id):
 
     try:
         ret, _ = cnode_client.disconnect_nqn(subsystem_nqn)
-    except:
+    except Exception:
         pass
     # if not ret:
     #     logger.error("failed to disconnect local connecting")
@@ -584,7 +568,7 @@ def deploy(ifname):
         ifname = "eth0"
 
     logger.info("Installing dependencies...")
-    ret = scripts.install_deps()
+    scripts.install_deps()
 
     dev_ip = utils.get_iface_ip(ifname)
     if not dev_ip:
@@ -592,23 +576,17 @@ def deploy(ifname):
         return False
 
     logger.info(f"Node IP: {dev_ip}")
-    ret = scripts.configure_docker(dev_ip)
+    scripts.configure_docker(dev_ip)
 
     node_docker = docker.DockerClient(base_url=f"tcp://{dev_ip}:2375", version="auto", timeout=60 * 5)
     # create the api container
-    nodes = node_docker.containers.list(all=True)
-    for node in nodes:
-        if node.attrs["Name"] == "/CachingNodeAPI":
-            logger.info("CachingNodeAPI container found, removing...")
-            node.stop()
-            node.remove(force=True)
-            time.sleep(2)
+    utils.remove_container(node_docker, '/CachingNodeAPI')
 
     logger.info("Creating CachingNodeAPI container")
     cont_image = constants.SIMPLY_BLOCK_DOCKER_IMAGE
-    container = node_docker.containers.run(
+    node_docker.containers.run(
         cont_image,
-        "python simplyblock_web/caching_node_app.py",
+        "python simplyblock_web/node_webapp.py caching_kubernetes_node",
         detach=True,
         privileged=True,
         name="CachingNodeAPI",
@@ -625,8 +603,8 @@ def deploy(ifname):
             f"DOCKER_IP={dev_ip}"
         ]
     )
-    logger.info("Pulling SPDK images")
-    node_docker.images.pull(constants.SIMPLY_BLOCK_DOCKER_IMAGE)
+    logger.info(f"Pulling SPDK image {constants.SIMPLY_BLOCK_SPDK_CORE_IMAGE}")
+    pull_docker_image_with_retry(node_docker, constants.SIMPLY_BLOCK_SPDK_CORE_IMAGE)
     return f"{dev_ip}:5000"
 
 
@@ -720,8 +698,10 @@ def remove_node(node_id, force=False):
         snode_api = CNodeClient(snode.api_endpoint)
         results, err = snode_api.spdk_process_kill()
         ret = snode_api.delete_dev_gpt_partitions(snode.nvme_devices[0].pcie_address)
+        if not ret:
+            logger.warning("Failed to delete gpt partitions, continuing anyway")
 
-    except:
+    except Exception:
         pass
 
     snode.remove(db_controller.kv_store)
