@@ -16,7 +16,7 @@ def _task_event(task, message, caused_by, event):
         db_object=task,
         caused_by=caused_by,
         message=message,
-        node_id=task.uuid,
+        node_id=task.node_id or task.uuid,
         status=task.status)
 
 
@@ -25,7 +25,8 @@ def task_create(task, caused_by=ec.CAUSED_BY_CLI):
 
 
 def task_updated(task, caused_by=ec.CAUSED_BY_CLI):
-    _task_event(task, f"Task updated: {task.uuid}", caused_by, ec.EVENT_STATUS_CHANGE)
+    return
+    # _task_event(task, f"Task updated: {task.uuid}", caused_by, ec.EVENT_STATUS_CHANGE)
 
 
 def task_canceled(task, caused_by=ec.CAUSED_BY_CLI):
